def is_win32():
    import sys
    return "win32" in sys.platform


def is_linux():
    import sys
    return "linux" in sys.platform


def execute(command, callback=None):
    import subprocess
    if not is_win32():
        command = ["/bin/sh", "-c", command]
        shell = False
    else:
        shell = True
    try:
        pipe = subprocess.Popen(
            command, shell=shell, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    except Exception as e:
        print e, command
        return e, -1

    if not callback:
        res = pipe.stdout.read()
        pipe.communicate()
        return (res, pipe.returncode)

    line = ""
    while True:
        char = pipe.stdout.read(1)
        if not char:
            break
        if char not in ["\n", "\r"]:
            line += char
            continue
        callback(line)
        line = ""
    pipe.communicate()
    return (None, pipe.returncode)



def local_ip():
    import socket
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        # doesn't even have to be reachable
        s.connect(('10.255.255.255', 0))
        ip = s.getsockname()[0]
    except Exception:
        ip = '127.0.0.1'
    finally:
        s.close()
    return ip


def html(url, **kwargs):
    import value
    import requests

    kwargs = value.AttrDict(kwargs)
    timeout = 60
    if kwargs.timeout:
        timeout = kwargs.timeout

    user_agent = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:52.0) Gecko/20100101 Firefox/52.0"

    if kwargs.headers:
        headers = kwargs.headers
    else:
        headers = value.AttrDict()

    if not headers["User-Agent"]:
        headers["User-Agent"] = user_agent

    try:
        res = requests.get(url, timeout=timeout, headers=headers)
    except Exception:
        return None

    return res.text


def soup(url, **kwargs):
    from bs4 import BeautifulSoup as bs

    text = html(url, **kwargs)
    if not text:
        return None

    soup = bs(text, "html.parser")
    return soup


def whois(address):
    import value
    import requests
    result = value.AttrDict()
    result.address = address

    api = "https://wq.apnic.net/whois-search/query"
    params = {
        "searchtext": address,
    }
    try:
        res = requests.get(api, params=params, timeout=5)
        items = res.json()
    except Exception:
        return result

    result.comments = []
    result.objects = []
    for item in items:
        item = value.AttrDict(**item)
        if item.type == "comments":
            result.comments.append(item)
        else:
            result.objects.append(item)
    return result



import error
import query
import system
import traffic
import value
import utils

__all__ = [
    "error",
    "query",
    "system",
    "traffic",
    "value",
    "utils",
]

__version__ = ".".join(
    [str(var) for var in
        [
            0,
            0,
            5,
            0, ]
     ])
